#include <gint/keyboard.h>
#include <gint/display.h>
#include <gint/timer.h>
#include <string.h>

#include "dungeon.h"
#include "save.h"

#define debug(format, args...) \
	dclear(C_WHITE); \
	dprint(1, 1, format, args); \
	dupdate(); \
	getkey();

//---
//	Main menu.
//---

/*
	level_select()
	Displays a screen where the user may select a level. Returns the level
	number on success, -1 if the user leaves the interface.
*/
int level_select(void)
{
	extern font_t res_font_arena;
	extern bopti_image_t res_locked;
	extern bopti_image_t
		res_wall0,
		res_wall1,
		res_wall2;
	bopti_image_t *bg[] = {
		&res_wall0,
		&res_wall1,
		&res_wall2,
	};
	const char *names[] = {
		"TUTORIAL",
		"PRISON",
		"COLOSSEUM",
	};
	int current = 0, total = 3;
	int i, redraw = 1, x, len;
	int conquered = save()->dungeons_conquered;

	while(1)
	{
		dclear(C_WHITE);

		if(current <= conquered) dimage(0, 0, bg[current]);
		else
		{
			dimage(40, 18, &res_locked);
			dtext(53, 22, C_BLACK, "LOCKED");
		}

		for(i = 0; i < total; i++)
		{
			x = 64 - (11 * total - 9) / 2 + 11 * i;
			dpixel(x, 58, C_BLACK);
			dpixel(x + 1, 58, C_BLACK);
			dpixel(x, 59, C_BLACK);
			dpixel(x + 1, 59, C_BLACK);

			if(i != current) continue;

			dline(x - 2, 56, x + 3, 56, C_BLACK);
			dline(x - 2, 56, x - 2, 61, C_BLACK);
			dline(x + 3, 56, x + 3, 61, C_BLACK);
			dline(x - 2, 61, x + 3, 61, C_BLACK);
		}

		len = 6 * strlen(names[current]) - 1;
		x = 64 - (len >> 1);

		drect(x - 1, 46, x + len, 52, C_WHITE);
		dfont(&res_font_arena);
		dtext(x, 47, C_BLACK, names[current]);

		dupdate();
		redraw = 0;

		volatile int timeout = 1;
		while(!redraw) switch(getkey_opt(GETKEY_NONE, &timeout).key)
		{
		case KEY_SHIFT:
			if(current <= conquered) return current;
			break;
		case KEY_ALPHA:
			return -1;
		case KEY_LEFT:
			if(current) current--, redraw = 1;
			break;
		case KEY_RIGHT:
			if(current < total - 1) current++, redraw = 1;
			break;
		}
	}

	// This point should be dead.
	return -1;
}

/*
	main_menu()
	Animates a main menu a displays various options with a moving cursor.
*/
int main_menu(void)
{
	extern font_t res_font_arena;
	extern bopti_image_t
		res_menu_background,
		res_menu_torches,
		res_menu_cursor;
	int frame = 0, frame_sub = 0;
	static int selected = 0;

	int cursor = 29 + 9 * selected;
	int cursor_next = cursor;

	dfont(&res_font_arena);

	volatile int timeout = 0;
	int t = timer_configure(TIMER_ANY, 1000000/32, GINT_CALL_SET(&timeout));
	timer_start(t);

	while(1)
	{
		// Drawing.

		dclear(C_WHITE);
		dimage(0, 0, &res_menu_background);
		dimage(17, cursor, &res_menu_cursor);
		dtext(52, 30, C_BLACK, "PLAY");
		dtext(43, 39, C_BLACK, "CREDITS");
		dtext(52, 48, C_BLACK, "QUIT");
		dsubimage(26, 1, &res_menu_torches, 8 * frame, 0, 7,
			23, DIMAGE_NONE);
		dsubimage(94, 1, &res_menu_torches,
			8 * ((frame + 2) % 4), 0, 7, 23, DIMAGE_NONE);
		dupdate();

		// Event handling.

		int key = getkey_opt(GETKEY_NONE, &timeout).key;
		if(timeout) {
			if(++frame_sub == 8) {
				frame = (frame + 1) % 4;
				frame_sub = 0;
			}
			cursor_next = 29 + 9 *selected;
			if(cursor != cursor_next)
				cursor += (cursor < cursor_next) ? (1) : (-1);
		}
		timeout = 0;

		switch(key)
		{
		case KEY_EXE:
		case KEY_SHIFT:
			timer_stop(t);
			return selected;

		case KEY_ALPHA:
		case KEY_MENU:
		case KEY_EXIT:
			timer_stop(t);
			return -1;

		case KEY_UP:
			if(selected) selected--;
			break;
		case KEY_DOWN:
			if(selected < 2) selected++;
			break;
		}
	}

	timer_stop(t);
}

/*
	credits()
	Displays a minimalist credit screen in the style of the main menu.
*/
void credits(void)
{
	extern font_t res_font_arena;
	extern bopti_image_t
		res_menu_background,
		res_menu_torches,
		res_menu_cursor;
	int frame = 0, key, i;

	const char *lines[] = {
		"BY LEPHENIXNOIR",
		"7 DAYS CPC #20",
		"PLANET-CASIO.COM",
		NULL
	};

	dfont(&res_font_arena);

	volatile int timeout = 0;
	int t = timer_configure(TIMER_ANY, 1000000/4, GINT_CALL_SET(&timeout));
	timer_start(t);

	while(1)
	{
		dclear(C_WHITE);
		dimage(0, 0, &res_menu_background);
		for(i = 0; lines[i]; i++)
			dtext(64 - ((6 * strlen(lines[i]) - 1) >> 1),
			30 + 9 * i, C_BLACK, lines[i]);
		dsubimage(26, 1, &res_menu_torches, 8 * frame, 0, 7,
			23, DIMAGE_NONE);
		dsubimage(94, 1, &res_menu_torches,
			8 * ((frame + 2) % 4), 0, 7, 23, DIMAGE_NONE);
		dupdate();

		key = getkey_opt(GETKEY_NONE, &timeout).key;
		if(timeout) frame = (frame + 1) % 4;
		timeout = 0;

		if(key == KEY_ALPHA || key == KEY_MENU || key == KEY_EXIT ||
			key == KEY_SHIFT || key == KEY_EXE) break;
	}

	timer_stop(t);
}



//---
//	Main function (execution router).
//---

int main(void)
{
	int x;

	save_load();

	while(1) switch(main_menu())
	{
	case -1:
	case 2:
		save_write();
		return 0;

	case 0:
		x = level_select();
		if(x == -1) break;
		if(dungeon(x)) credits();
		save_write();
		break;

	case 1:
		credits();
		break;
	}

	save_write();
	return 0;
}
