#include "save.h"
#include <gint/bfile.h>
#include <gint/gint.h>
#include <string.h>

static struct Save save_player;
static const char *carrus_str = "CARRUS";
static const unsigned short filename[] = {
	'\\', '\\', 'f', 'l', 's', '0', '\\',
	'a', 'r', 'e', 'n', 'a', '.', 's', 'a', 'v', 0x00
};

/*
	save()
	Returns the save data address.
*/
struct Save *save(void)
{
	return &save_player;
}

/*
	save_loadFresh()
	Loads a new game into the save data.
*/
static void save_loadFresh(void)
{
	save_player.entity.name		= carrus_str;
	save_player.entity.max_hp	= 80;
	save_player.entity.max_mp	= 12;
	save_player.entity.atk		= 12;
	save_player.entity.def		= 4;
	save_player.entity.spe		= 3;

	save_player.entity.skills[0]	= Skill_None;
	save_player.entity.skills[1]	= Skill_None;
	save_player.entity.skills[2]	= Skill_None;

	save_player.dungeons_conquered	= 0;
}

/*
	save_write()
	Saves to a file.
*/
static void save_write_os(void)
{
	BFile_Remove(filename);
	int file_size = sizeof(save_player);
	BFile_Create(filename, BFile_File, &file_size);
	int fd = BFile_Open(filename, BFile_WriteOnly);
	BFile_Write(fd, &save_player, sizeof(save_player));
	BFile_Close(fd);
}
void save_write(void)
{
	gint_world_switch(GINT_CALL(save_write_os));
}

/*
	save_load()
	Loads from a file. If the file doesn't exist, loads the new game data
	and saves.
*/
static int save_load_os(void)
{
	int fd = BFile_Open(filename, BFile_ReadOnly);

	if(fd < 0)
	{
		if(fd != -1) return fd;
		int file_size = sizeof(save_player);
		BFile_Create(filename, BFile_File, &file_size);

		save_loadFresh();
		save_write_os();
	}
	else
	{
		BFile_Read(fd, &save_player, sizeof(save_player), 0);
		BFile_Close(fd);
	}

	return 0;
}
int save_load(void)
{
	return gint_world_switch(GINT_CALL(save_load_os));
}
